C
      PROGRAM MPOLAR
C.........................................................................
C
C     This is an MSES or MSIS driver which sweeps over a range of angles 
C     of attack of an isolated airfoil, thus generating a polar curve.
C
C     The calculated polar itself is written to two files:
C          polar.xxx   (formatted,   unit 4 )
C          polarx.xxx  (unformatted, unit 11)
C
C     polar.xxx contains only the integrated airfoil forces and transition
C     locations.  polarx.xxx also contains these, plus the surface 
C     pressures and BL parameters as functions of x/c for each point, 
C     and also the airfoil coordinates for pretty-plotting purposes.
C
C     If both of the two files already exist, AND represent the same 
C     airfoil and freestream conditions, then the new points are 
C     appended to them.
C
C     If both of the two files already exist, and DO NOT represent the
C     same airfoil and freestream conditions, then POLAR will terminate
C     without calculating any operating points.  This is to prevent
C     clobbering of any existing polar disk files.
C
C     Normally, MPOLAR tries to converge each specified ALFA value.  
C     If convergence fails or an underrelaxation factor less than RLXMIN
C     appears, then the ALFA increment from the last converged point
C     is cut in half and convergence is attempted for that point.
C
C     Each time a point converges successfully, MDAT.xxx is overwritten
C     with the converged solution.  Hence, if all the following points 
C     bomb, the user can run the following points "manually" with MSES 
C     or MSIS and perhaps figure out why they bombed (massive separation,
C     local Reynolds number too low, not enough dissipation, etc., etc.).
C
C..........................................................................
C
      PARAMETER (NAX=500)
      INCLUDE 'STATE.INC'
      INCLUDE 'MSES.INC'
C
      CHARACTER*80 FNAME3, FNAME4, FNAME11, ARGP1, ARGP2
      COMMON/FNAM/ FNAME3, FNAME4, FNAME11, ARGP1, ARGP2
C
      CHARACTER*80 LINE, FNAME, FMSES, FMDAT, FSENS
      CHARACTER*16 CPARM
      LOGICAL LSOLVE, ERROR
C
      REAL QS(IX), P(IX)
C
      REAL AINPUT(3)
      CHARACTER*1 DUMMY
C
      REAL SPEC(0:NAX)
      INTEGER KSAVE(0:NAX), NCUT(0:NAX)
      CHARACTER*1 CHRS
C
C
      PARAMETER(IPMAX=20,JPMAX=4)
      INTEGER IPOL(IPMAX), JPOL(JPMAX)
      REAL CPOL(NAX,IPMAX), CPOLSD(NAX,ISX,IPMAX)
C
      INCLUDE 'PINDEX.INC'
      INCLUDE 'EPS.INC'
C
 1000 FORMAT(A)
C
      CALL GETARG0(1,ARGP1)
      CALL GETARG0(2,ARGP2)
      IF(INDEX(ARGP2,' ').LE.1) ARGP2 = ARGP1
C
C---- max number of Newton iterations per point
      INEWT = 24
C
C---- max number of specified-quantity interval subdivisions
      NCUTMAX = 2

C---- minimum allowable underrelaxation factor (see header above)
      RLXMIN = 0.03
C
C---- set indices of parameters to be written out  (see  INC/PINDEX.INC )
C-    note: list might be extended farther below
      IPOL( 1) = IAL  ! alpha
      IPOL( 2) = ICL  ! CL
      IPOL( 3) = ICD  ! CD
      IPOL( 4) = ICM  ! CM
      IPOL( 5) = ICW  ! CDwave
      IPOL( 6) = ICV  ! CDvisc
      IPOL( 7) = ICP  ! CDpres
      IPOL( 8) = IMA  ! Mach
      IPOL( 9) = IRE  ! Re
      NIPOL = 9
C
      JPOL(1) = JTN  ! transition location
      NJPOL = 1
C
C
C---- degrees to radians factor
      DTOR = ATAN(1.0)/45.0
C
C---- set machine epsilon
      CALL EPSSET(EPSILON)
C
C---- read MSES state file
      FMDAT = 'mdat.' // ARGP1
      LU = 1
      CALL INPUT(LU,FMDAT)
C
      FNAME3  = 'spec.'   // ARGP1
      FNAME4  = 'polar.'  // ARGP2
      FNAME11 = 'polarx.' // ARGP2
C
C==========================================================================
C---- read specified-parameter file
      LU3 = 3
      OPEN(LU3,FILE=FNAME3,STATUS='OLD',FORM='FORMATTED')
C
C---- read specified-quantity indicator
      IA = 1
      READ(LU3,*,END=11) KSPEC
C
C---- read in specified quantities
      DO 10 IA = 1, NAX
CCC        READ(LU3,*,END=11) SPEC(IA)
        READ(LU3,1000,END=11) LINE
        NINP = 2
        CALL GETFLT(LINE,AINPUT,NINP,ERROR)
C
C------ exit on blank line
        IF(NINP.EQ.0) GO TO 11
C
        SPEC(IA) = AINPUT(1)
        NCUT(IA) = 0
C
        IF(NINP.GE.2) THEN
         KSAVE(IA) = INT(AINPUT(2))
         IF(KSAVE(IA).LT.1 .OR. KSAVE(IA).GT.99) KSAVE(IA) = 0
        ELSE
         KSAVE(IA) = 0
        ENDIF
 10   CONTINUE
      WRITE(*,*) 'MPOLAR: Array limit reached.  Increase NAX'
C
 11   NA = IA-1
      CLOSE(LU3)
C==========================================================================
C
C---- save name and current value of parameter to be specified (may be changed by INIT)
      CL = LIFT / (0.5*RHOINF*QINF**2)
      SQCL = SQRT( ABS(CL) )
      IF    (KSPEC.EQ. 5) THEN
       CPARM = 'alpha:'
       SCURR = ALFA/DTOR
      ELSEIF(KSPEC.EQ. 6) THEN
       CPARM = 'CL:'
       SCURR = CL
      ELSEIF(KSPEC.EQ.15) THEN
       CPARM = 'Mach:'
       SCURR = MINF
      ELSEIF(KSPEC.EQ.16) THEN
       CPARM = 'sqrt(CL)*Mach:'
       SCURR = MINF * SQCL
      ELSEIF(KSPEC.EQ.17) THEN
       CPARM = 'Re:'
       SCURR = REYN * RHOINF*QINF/MUINF
      ELSEIF(KSPEC.EQ.18) THEN
       CPARM = 'sqrt(CL)*Re:'
       SCURR = REYN * RHOINF*QINF/MUINF * SQCL
      ELSE
       WRITE(*,*) 'Illegal specified-quantity index.'
       WRITE(*,*) 'Currently legal indices:  5 6 15 16 17 18'
       STOP
      ENDIF
C
C---- initialize MSES for solution calculations
      CALL INIT
C
C---- extend the polar-variable list if appropriate
      IF(ISDELH .NE. 0) THEN
       IP = NIPOL
C
       IP = IP+1
       IPOL(IP) = ICDH
C
       IP = IP+1
       IPOL(IP) = ICMDOT
C
       NIPOL = IP
      ENDIF
C
C---- make sure that one of the global constraints matches KSPEC
      DO KGC = 1, NGCON
        IF(KSPEC.EQ.KGCON(KGC)) GO TO 16
      ENDDO
      WRITE(*,1510) KSPEC
 1510 FORMAT(/1X,'Must specify global constraint', I3, '  in mses.xxx')
      STOP
 16   CONTINUE
C
C
C---- set up listing and dump files
      CALL PINIT(NAX,NAINIT,CPOL,CPOLSD,IPOL,NIPOL,JPOL,NJPOL)
      CALL PXINIT
C
C---- set "zeroth" prescribed parameter for subdividing first interval
      IF(NA.GT.1) SPEC(0) = 2.0*SPEC(1) - SPEC(2)
      IF(NA.EQ.1) SPEC(0) = SPEC(1)
C
      IABEG = 1
C
      IF(NAINIT.EQ.0) THEN
C----- no old polar... just go start new one
       SLAST = -999.0
       GO TO 24
C
      ELSE
C----- set last specified-parameter value in stored polar
       IA = NAINIT
       IF    (KSPEC.EQ. 5) THEN
        SLAST = CPOL(IA,IAL)
       ELSEIF(KSPEC.EQ. 6) THEN
        SLAST = CPOL(IA,ICL)
       ELSEIF(KSPEC.EQ.15) THEN
        SLAST = CPOL(IA,IMA)
       ELSEIF(KSPEC.EQ.16) THEN
        SLAST = CPOL(IA,IMA) * SQRT(ABS(CPOL(IA,IAL)))
       ELSEIF(KSPEC.EQ.17) THEN
        SLAST = CPOL(IA,IRE)
       ELSEIF(KSPEC.EQ.18) THEN
        SLAST = CPOL(IA,IRE) * SQRT(ABS(CPOL(IA,IAL)))
       ENDIF
C
      ENDIF
C
C---- if this is a fresh mdat.xxx, go start from beginning
      IF(ICOUNT.EQ.0) GO TO 24
C
C---- find which SPEC to begin sweep at
      DO 20 IABEG = 1, NA
        SPECM = SPEC(IABEG-1)
        SPECO = SPEC(IABEG  )
C
C------ current SCURR is nearly same as SPECO, go do SPECO
        IF(ABS(SPECO-SCURR) .LT. EPSM) GO TO 24
C
C------ current SCURR is between SPECO and SPECM, go do SPECO
        SGN = SIGN( 1.0 , SPECO-SPECM )
        IF( SGN*SPECM .LE. SGN*SCURR .AND. 
     &      SGN*SPECO .GT. SGN*SCURR       ) GO TO 24
 20   CONTINUE
      IABEG = 1
C
C---- if the last SPEC in polar.xxx is same as current SCURR, do next one
 24   IF(ABS(SLAST-SCURR) .LT. EPSM) IABEG = IABEG+1
C
C---- confirm
      WRITE(*,*) ' '
      WRITE(*,*) 'Prescribed ', CPARM
      DO 28 IA = 1, NA
        IF(IA.EQ.IABEG) THEN
         WRITE(*,9005) IA, SPEC(IA),PNUM(KSAVE(IA)),'   (start)'
        ELSE
         WRITE(*,9005) IA, SPEC(IA),PNUM(KSAVE(IA))
        ENDIF
 9005   FORMAT(3X,I3,F10.5,3X,A2, A10)
 28   CONTINUE
      WRITE(*,*)
C
C---- if last point in spec.xxx was already computed, just stop
      IF(IABEG.GT.NA) THEN
       STOP
      ENDIF
C
      LCONV = .FALSE.
C
      IA = IABEG
C
C================================================================
C---- top of specified-quantity loop
 100  CONTINUE
C
      SPECIN = SPEC(IA)
      WRITE(*,*)
      WRITE(*,*) 'Specified parameter: ',SPECIN  ! , ncut(ia), ksave(ia)
cc      pause
C
      IF    (KSPEC.EQ. 5) THEN
       ALFAIN = SPECIN*DTOR
      ELSEIF(KSPEC.EQ. 6) THEN
       CLIFIN = SPECIN
      ELSEIF(KSPEC.EQ.15) THEN
       MACHIN = SPECIN
      ELSEIF(KSPEC.EQ.16) THEN
       MACHIN = SPECIN
      ELSEIF(KSPEC.EQ.17) THEN
       REYNIN = SPECIN
      ELSEIF(KSPEC.EQ.18) THEN
       REYNIN = SPECIN
      ENDIF
C
      LSMOVE = .FALSE.
C
C---- if inverted RHSs exist and ALFA is a global variable, then...
      IF(LCONV) THEN
C
C----- clear residual-driven changes (should be nearly zero anyway)
       CALL CLRRHS(1)
C
C----- free Newton iteration (!) via existing  d()/dSPEC  RHS vector
C-     and new  (SPECIN - SPEC)  global residual driving term
       CALL UPDATE(.TRUE.)
C
       IF(LSMOVE) CALL SMOVE(-4,AINF,0.4*RHOINF)
      ENDIF
C
C---- set Newton iteration limit
      NITER1 = INEWT
C
C-----allow for 6 extra iterations for first point
      IF(IA.EQ.IABEG) NITER1 = NITER1 + 6
C
C---- Newton iteration loop
      DO 40 ITER = 1, NITER1
        NITER = ITER + 1
C
 39     ICOUNT = ICOUNT + 1
C
C------ set up mass fraction arrays
        CALL MFCALC
C
C------ calculate grid movement direction vectors at all grid nodes
        CALL NCALC
C
C------ calculate unit vectors for displacement thickness offsetting
        CALL BLNORM
C
C------ set geometry sensitivities
        CALL GEOSEN
C
C------ set geometric mode shapes
        IF(LMODI .OR. LMINV) THEN
         CALL GNSET
        ENDIF
C
C------ set element position modes and linearize grid spacing arrays
        IF(LPOSI .OR. LPINV) THEN
         CALL PNSET
         CALL SGLIN
        ENDIF
C
C------ set up actuator-disk enthalpy jumps, if any
        CALL SETHT
C
C------ set dissipation augmentation from current residuals
        DISS = 0.
        CALL SETUP0
        DISS = RDISS
C
C------ calculate residuals and Jacobian entries at all interior nodes
        CALL SETUP
C
        LVISC = REYNIN.GT.0.0
C
C------ calculate BL residuals & Jacobians, and combine with main system
        IF(LVISC) CALL SETBL
C
C------ calculate BC residuals & Jacobians, and combine with main system
        CALL SETBC
C
C----- solve main Newton system
        CALL SOLVE
        LSOLVE = .TRUE.
C
C------ update all variables and set LSMOVE flag
        CALL UPDATE(.TRUE.)
C
C------ fix up grid if necessary and do at least one more iteration
        IF(LSMOVE) THEN
         CALL SMOVE(-4,AINF,0.4*RHOINF)
         GO TO 40
        ENDIF
C

C------ use hardwired tolerances in EPS.INC, which is what MSES uses
c        EPSN = EPSNF*EPSILON*100.0
c        EPSR = EPSRF*EPSILON*100.0
c        EPSV = EPSVF*EPSILON*10000.0
C
C------ set convergence flag
        LCONV = DNRMS  .LT. EPSN      .AND.
     &          DRRMS  .LT. EPSR      .AND.
     &          DVRMS  .LT. EPSV      .AND.
     &      ABS(DNMAX) .LT. EPSN*10.0 .AND.
     &      ABS(DRMAX) .LT. EPSR*10.0 .AND.
     &      ABS(DVMAX) .LT. EPSV*10.0
C
C
C------ check for solution blowing up -- if so, subdivide interval
        IF(RLX.LT.RLXMIN .AND. IA.GT.1) GO TO 41
C
        IF(LCONV) THEN
C------- we converged... finish up
C
         IF((LMODI.OR.LPOSI) .AND. ITER.LT.NITER .AND.
     &       KSAVE(IA).NE.0 .AND. NCUT(IA).EQ.0        ) THEN
C--------- Optimization case: do one more iteration to get sensitivities
C-         (Note: if ITER>=NITER, then SOLVE inverts all righthand sides)
           NITER = ITER
           WRITE(*,*)
           WRITE(*,*) 'Final sensitivity iteration...'
           GO TO 39
         ENDIF
C
C------- we converged for this angle of attack
C
         DO 402 JO=1, JJ-1
           IF(JSTAG(JO).GT.0) GO TO 402
           JP = JO+1
           CALL PICALC(JO, Q(1,JO),QS,P, PI(1,JO),PI(1,JP))
 402     CONTINUE

C-------- these are called in GLOSEN   MD  24Sep09
c         CALL LDCALC
c         CALL DVCALC
c         CALL DWCALC
C
         CALL GLOSEN
C
         IF(KSAVE(IA).NE.0 .AND. NCUT(IA).EQ.0) THEN
C-------- also save this point to mdat.xxx_nn state file
          K = INDEX(FMDAT,' ') - 1
          FNAME = FMDAT(1:K) // '_' // PNUM(KSAVE(IA))
          LU = 1
          WRITE(*,*) 'Writing file  ', FNAME(1:K+3),'  ...'
          CALL OUTPUT(LU,FNAME)
C
C-------- also generate a mses.xxx_nn input file
          FMSES = 'mses.' // ARGP1
          K = INDEX(FMSES,' ') - 1
          FNAME = FMSES(1:K) // '_' // PNUM(KSAVE(IA))
          LU = 2
          WRITE(*,*) 'Writing file  ', FNAME(1:K+3),'  ...'
          CALL MSWRIT(LU,FNAME, ERROR, NGLX, NBL,
     &                KGVAR,NGCON,
     &                KGCON,NGCON,
     &                MACHIN, CLIFIN, ALFAIN,
     &                ISMOM,IFFBC, DOUXIN,DOUYIN,SRCEIN,
     &                REYNIN, ACRIT, KTRTYP,
     &                XTR1,
     &                MCRIT, MUCON,
     &                ISMOVE, ISPRES,
     &                NMODN, NPOSN,
     &                ISDELH, XCDELH, PTRHIN, ETAH )
C
C-------- save total sensitivities to unformatted file
          IF(LMODI .OR. LPOSI) THEN
           FSENS = 'sensx.' // ARGP1
           K = INDEX(FSENS,' ') - 1
           FNAME = FSENS(1:K) // '_' // PNUM(KSAVE(IA))
           LU = 8
           WRITE(*,*) 'Writing file  ', FNAME(1:K+3),'  ...'
           CALL SNWRIT(LU,FSENS,KSAVE(IA))
          ENDIF
         ENDIF
C
C------- save state file as usual
         LU = 1
         CALL OUTPUT(LU,FMDAT)

         CL = LIFT/QU
         CM = MOMN/QU
         CDV = DRAGV/QU
         CDW = DRAGW/QU
         CDF = DRAGF/QU
         CD  = CDW + CDV
         CDP = CD  - CDF
         CDH   = DRAGH / QU
         CMDOT = MASSH / (RHOINF*QINF)
C
C------- write forces to formatted polar listing file and screen
         WRITE(*,*)
         WRITE(*,*) 'Writing out operating point . . . '
C
         WRITE(*,9100)
         WRITE(*,9110) ALFA/DTOR,MINF,CL,CD,CDW,CM,CDH,CMDOT,
     &                  (XTR(IS),IS=1,2*NBL)
 9100    FORMAT(1X,
     &    '  alpha    Mach      CL        CD       CDw       CM   ',
     &    '    CDh      Cmdot ',
     &    '  XtrTop  XtrBot')
 9110    FORMAT(1X,
     &       F7.3,    F9.4,    F9.4,    F10.5,    F10.5,    F9.4,
     &      F10.5,    F9.5,
     &     10( F9.4 ,  F8.4) )
C
         CALL PADD(NAX,IA,CPOL,CPOLSD,IPOL,NIPOL,JPOL,NJPOL)
         CALL PXADD
C
         IF(IA .GE. NA) THEN
          STOP
C
         ELSE
C-------- go calculate next IA point
          IA = IA + 1
          GO TO 100
         ENDIF
C
        ENDIF
C
 40   CONTINUE
      IF(IA .EQ. 1) STOP 'MPOLAR: Failed to converge first point'
C
 41   CONTINUE
C
C-----convergence failed -- subdivide current interval
C
      IF(NCUT(IA) .GE. NCUTMAX) THEN
C----- repeated subdivision didn't achieve convergence...
C-     skip all points at this cut level
       DO KA = IA+1, NA
         IF(NCUT(KA) .LT. NCUT(IA)) THEN
          IA = KA
          GO TO 100
         ENDIF
       ENDDO
C
C-     something is screwed up in a major way
       STOP 'MPOLAR: Severe convergence problem detected'
      ENDIF
C
      WRITE(*,*) 'Cutting interval ...'

      IF(IA.EQ.NA) THEN
C----- last point... set SPEC at 2/3 of current interval
       SPEC(NA) = SPEC(IA-1) + (SPEC(IA)-SPEC(IA-1))*0.6666667
       NCUT(NA) = NCUT(NA) + 1

      ELSE
C----- interior point... subdivide two intervals IA-1 IA IA+1 into three intervals,
C-     and extend SPEC array with the extra point (clip end at array limit)
       SPEC13 = SPEC(IA-1) + (SPEC(IA+1)-SPEC(IA-1))*0.3333333
       SPEC23 = SPEC(IA-1) + (SPEC(IA+1)-SPEC(IA-1))*0.6666667

       NANEW = MIN( NA+1 , NAX )
       DO KA = NANEW-1, IA, -1
         SPEC(KA+1) = SPEC(KA)
         NCUT(KA+1) = NCUT(KA)
         KSAVE(KA+1) = KSAVE(KA)
       ENDDO
       SPEC(IA  ) = SPEC13
       SPEC(IA+1) = SPEC23

       NCUT(IA  ) = NCUT(IA  ) + 1
       NCUT(IA+1) = NCUT(IA+1) + 1

       KSAVE(IA  ) = 0
       KSAVE(IA+1) = 0

       NA = NANEW
      ENDIF
C
C-----re-read old converged solution and try again...
CCC      REWIND 1
      LU = 1
      CALL INPUT(LU,FMDAT)
C
C---- go back with the same current IA index
      GO TO 100
C
      STOP
      END ! MPOLAR



      SUBROUTINE PINIT(NAX,NA,CPOL,CPOLSD,IPOL,NIPOL,JPOL,NJPOL)
      INCLUDE 'STATE.INC'
      INCLUDE 'MSES.INC'
      CHARACTER*80 FNAME3, FNAME4, FNAME11, ARGP1, ARGP2
      COMMON/FNAM/ FNAME3, FNAME4, FNAME11, ARGP1, ARGP2
C
      REAL CPOL(NAX,*), CPOLSD(NAX,ISX,*)
      INTEGER IPOL(*),JPOL(*)
C
      CHARACTER*32 NAMEX, CODEX
      REAL XTRIPX(ISX)
C
      LOGICAL ERROR, NAMDIF
C
      INCLUDE 'PINDEX.INC'
C
      IF(LDEPMA) THEN
       IMATYP = 2
      ELSE
       IMATYP = 1
      ENDIF
      IF(LDEPRE) THEN
       IRETYP = 2
      ELSE
       IRETYP = 1
      ENDIF
C    
C---- assume old polar won't exist
      NA = 0
C
C---- try reading the polar file to see if it exists
      LU4 = 4
      OPEN(LU4,FILE=FNAME4,STATUS='OLD',ERR=60)
      CALL POLREAD(LU4,' ',ERROR,
     &             NAX,NA,CPOL, 
     &             REYNX,MACHX,ACRITX,XTRIPX,PTRATX,ETAPX,
     &             NAMEX,IRETYPX,IMATYPX,
     &             ISX,NBLX,CPOLSD,
     &             CODEX,VERSX )
      IF(ERROR) GO TO 90
      CLOSE(LU4)
C
C---- check to see if this is the same airfoil
      DO K=1, 32
        IF(NAME(K:K).NE.NAMEX(K:K)) THEN
         WRITE(*,*) 
     &     '*** Existing polar listing file has a different NAME'
         NAMDIF = .TRUE.
         GO TO 21
        ENDIF
      ENDDO
      NAMDIF = .FALSE.
   21 CONTINUE
C
C---- check if the polar save file is for the same airfoil and conditions
      IF(NAMDIF                   .OR.
     &   REYNX     .NE. REYNIN   .OR.
     &   MACHX     .NE. MACHIN   .OR.
     &   IRETYPX   .NE. IRETYP   .OR.
     &   IMATYPX   .NE. IMATYP   .OR.
     &   ACRITX    .NE. ACRIT    .OR.
     &   XTRIPX(1) .NE. XTR1(1)  .OR.
     &   XTRIPX(2) .NE. XTR1(2)       ) THEN
C
       WRITE(*,6600)  NAME  ,  NAMEX   ,
     &                REYNIN,  REYNX   ,
     &                MACHIN,  MACHX   ,
     &                IRETYP,  IRETYPX ,
     &                IMATYP,  IMATYPX ,
     &                ACRIT ,  ACRITX  ,
     &                XTR1(1), XTRIPX(1),
     &                XTR1(2), XTRIPX(2)
C
 6600  FORMAT(
     & /'              Current                         Save file'
     & /'          ------------------              ------------------'
     & /' name :   ', A    ,      A
     & /' Re   :   ', F12.0, 20X, F12.0
     & /' Mach :   ', F12.4, 20X, F12.4
     & /' Retyp:   ', I7   , 25X, I7
     & /' Matyp:   ', I7   , 25X, I7
     & /' Ncrit:   ', F12.4, 20X, F12.4
     & /' xtr T:   ', F12.4, 20X, F12.4
     & /' xtr B:   ', F12.4, 20X, F12.4 )
C
       WRITE(*,*)
       WRITE(*,*)
     &    'Current parameters different from old save file values.'
      ENDIF
C
C---- display polar save file just read in
      WRITE(*,*)
      WRITE(*,*) 'Old polar save file read in ...'
      CALL POLWRIT(6,' ',ERROR, .TRUE.,
     &             NAX, 1,NA, CPOL, IPOL,NIPOL,
     &             REYNX,MACHX,ACRITX,XTRIPX,PTRATX,ETAPX,
     &             NAMEX, IRETYPX,IMATYPX,
     &             ISX,1,CPOLSD, JPOL,NJPOL,
     &             CODEX,VERSX, .FALSE. )
C
      RETURN
C
C---- the polar save file doesn't exist, so write new header
   60 CONTINUE
C
      OPEN(LU4,FILE=FNAME4,STATUS='NEW',ERR=80)
      CALL POLWRIT(LU4,' ',ERROR, .TRUE.,
     &             NA, 0,-1, CPOL, IPOL,NIPOL,
     &             REYNIN,MACHIN,ACRIT,XTR1,PTRHIN,ETAH,
     &             NAME,IRETYP,IMATYP,
     &             ISX,1,CPOLSD, JPOL,NJPOL,
     &             'MSES',VERSION, .FALSE. )
      CLOSE(LU4)
C
      ERROR = .FALSE.
      RETURN
C
 80   CONTINUE
      ERROR = .TRUE.
C
 90   CONTINUE
      RETURN
      END ! PINIT



      SUBROUTINE PXINIT
      INCLUDE 'STATE.INC'
      INCLUDE 'MSES.INC'
      CHARACTER*80 FNAME3, FNAME4, FNAME11, ARGP1, ARGP2
      COMMON/FNAM/ FNAME3, FNAME4, FNAME11, ARGP1, ARGP2
C
      CHARACTER*32 NAMEX
      INTEGER ILEX(NBX), ITEX(NBX), IIBX(NBX)
C
      IF(LDEPMA) THEN
       IMATYP = 2
      ELSE
       IMATYP = 1
      ENDIF
      IF(LDEPRE) THEN
       IRETYP = 2
      ELSE
       IRETYP = 1
      ENDIF
C
C---- try reading the unformatted polar dump file to see if it exists
      LU11 = 11
      OPEN(LU11,FILE=FNAME11,STATUS='UNKNOWN',FORM='UNFORMATTED')
      READ(LU11,ERR=56,END=56) NAMEX
C
      WRITE(*,*) 'Will append to existing polar dump file'
C
C---- if we got to here, it exists, so read the header
      READ(LU11) MACHX, REYNX, ACRITX
      READ(LU11) IMATYPX, IRETYPX
      READ(LU11) NBLX, IIX
      READ(LU11) (ILEX(N), ITEX(N), IIBX(N), N=1, NBLX)
C
C---- check to see if this is the same case
      DO 50 K=1, 32
        IF(NAME(K:K).NE.NAMEX(K:K)) THEN
         WRITE(*,*) 
     &     'WARNING: Existing polar dump file has a different NAME'
         GO TO 51
        ENDIF
   50 CONTINUE
   51 CONTINUE
C
      IF(MACHIN.NE.MACHX) WRITE(*,*) 
     & '*** Existing polar dump file has a different Mach number'
      IF(REYNIN/1.0E6.NE.REYNX) WRITE(*,*) 
     & '*** Existing polar dump file has a different Reynolds number'
      IF(ACRIT.NE.ACRITX) WRITE(*,*) 
     & '*** Existing polar dump file has a different Ncrit'
      IF(IRETYP .NE. IRETYPX) WRITE(*,*)
     & '*** Existing polar listing file has different Re(CL) dependence'
      IF(IMATYP .NE. IMATYPX) WRITE(*,*)
     & '*** Existing polar listing file has different M(CL) dependence'
      IF(II.NE.IIX)
     & STOP 'Existing polar dump file has a different grid i-size'
      DO 53 N=1, NBLX
        IF(ILEB(N).NE.ILEX(N) .OR. ITEB(N).NE.ITEX(N))
     & STOP 'Existing polar dump file has different LE or TE grid index'
   53 CONTINUE
C
C---- set polar dump file pointer at the end
   55 READ(LU11,END=60) DUMMY
      GO TO 55
C
C
C---- the polar dump file doesn't exist, so write new header
   56 CONTINUE
      WRITE(LU11) NAME, ' MSES   ', VERSION
      WRITE(LU11) MACHIN, REYNIN/1.0E6, ACRIT
      WRITE(LU11) IMATYP, IRETYP
      WRITE(LU11) NBL, II
      WRITE(LU11) (ILEB(N), ITEB(N), IIB(N), N=1, NBL)
      DO 59 N=1, NBL
        WRITE(LU11) (XB(IB,N), YB(IB,N), IB=1, IIB(N))
   59 CONTINUE
C
   60 CONTINUE
C
      CLOSE(LU11)
      RETURN
      END ! PXINIT



      SUBROUTINE PADD(NAX,IA,CPOL,CPOLSD,IPOL,NIPOL,JPOL,NJPOL)
      INCLUDE 'STATE.INC'
      INCLUDE 'MSES.INC'
      CHARACTER*80 FNAME3, FNAME4, FNAME11, ARGP1, ARGP2
      COMMON/FNAM/ FNAME3, FNAME4, FNAME11, ARGP1, ARGP2
C
      REAL CPOL(NAX,*), CPOLSD(NAX,ISX,*)
      INTEGER IPOL(*),JPOL(*)
C
      LOGICAL ERROR
C
      INCLUDE 'PINDEX.INC'
C
      DTOR = ATAN(1.0)/45.0
C
      CL = LIFT/QU
      CM = MOMN/QU
      CDV = DRAGV/QU
      CDW = DRAGW/QU
      CDF = DRAGF/QU
      CD  = CDW + CDV
      CDP = CD  - CDF
C
      CDH   = DRAGH / QU
      CMDOT = MASSH / (RHOINF*QINF)
      REINF = REYN * (RHOINF*QINF/MUINF)
C
      CPOL(IA,IAL) = ALFA/DTOR
      CPOL(IA,ICL) = CL
      CPOL(IA,ICD) = CD
      CPOL(IA,ICM) = CM
      CPOL(IA,ICW) = CDW
      CPOL(IA,ICV) = CDV
      CPOL(IA,ICP) = CDP
      CPOL(IA,IMA) = MINF
      CPOL(IA,IRE) = REYN * (RHOINF*QINF/MUINF)
      CPOL(IA,INC) = ACRIT
C
      CPOL(IA,ICDH)   = CDH
      CPOL(IA,ICMDOT) = CMDOT
C
      DO IS = 1, 2*NBL
        IF(LVISC) THEN
         XOCT = XTR(IS)
        ELSE
         XOCT = 0.
        ENDIF
        CPOLSD(IA,IS,JTP) = XTR1(IS)
        CPOLSD(IA,IS,JTN) = XOCT
      ENDDO
C
      LU = 4
CCC   OPEN(LU,FILE=FNAME4,STATUS='OLD',FORM='FORMATTED')   ! %%% RS/6000
CCC   OPEN(LU,FILE=FNAME4,STATUS='OLD',
CCC  &          ACCESS='SEQUENTIAL',POSITION='APPEND')     ! %%% Fortran 90/95
      OPEN(LU,FILE=FNAME4,STATUS='OLD',
     &          FORM='FORMATTED',ACCESS='APPEND')
      CALL POLWRIT(LU,' ',ERROR, .FALSE.,
     &             NAX, IA,IA, CPOL, IPOL,NIPOL,
     &             REINF,MINF,ACRIT,XTR1,PTRHIN,ETAH,
     &             NAME, IRETYP,IMATYP,
     &             ISX,NBL,CPOLSD, JPOL,NJPOL,
     &             CODE,VERSION, .FALSE. )
      CLOSE(LU)
C
      RETURN
      END ! PADD



      SUBROUTINE PXADD
      INCLUDE 'STATE.INC'
      INCLUDE 'MSES.INC'
C      
      CHARACTER*80 FNAME3, FNAME4, FNAME11, ARGP1, ARGP2
      COMMON/FNAM/ FNAME3, FNAME4, FNAME11, ARGP1, ARGP2
C
      REAL XX(IX,ISX), CP(IX,ISX), CF(IX,ISX)
C
      DTOR = ATAN(1.0)/45.0
C
      CL = LIFT/QU
      CM = MOMN/QU
      CDV = DRAGV/QU
      CDW = DRAGW/QU
      CDF = DRAGF/QU
      CD  = CDW + CDV
      CDP = CD  - CDF
C
      CDH   = DRAGH / QU
      CMDOT = MASSH / (RHOINF*QINF)
      REINF = REYN * (RHOINF*QINF/MUINF)
C
C---- write forces and surface variables to unformatted dump file
      LU = 11
CCC   OPEN(LU,FILE=FNAME11,STATUS='OLD',FORM='UNFORMATTED') !%%% RS/6000
CCC   OPEN(LU,FILE=FNAME11,STATUS='OLD',
CCC  &          ACCESS='SEQUENTIAL',POSITION='APPEND')     ! %%% Fortran 90/95
      OPEN(LU,FILE=FNAME11,STATUS='OLD',
     &           FORM='UNFORMATTED',ACCESS='APPEND')
      WRITE(LU) ALFA/DTOR,MINF,REINF,CL,CD,CDW,CM,CDH,CMDOT,
     &          (XTR(IS),IS=1,2*NBL)
C
C---- for each surface...
      DO 404 IS=1, 2*NBL
        N = (IS+1)/2
C
        IF(MOD(IS,2).EQ.1) THEN
         STOT = SB(1     ,N) - SBLE(N)
         JS = JS1(N)
        ELSE
         STOT = SB(IIB(N),N) - SBLE(N)
         JS = JS2(N)
        ENDIF
C
C------ ... write out Cp's and BL parameters to unformatted dump file
        DPINF = PINF - PSTINF
        DO 4045 I=1, II
          IG = I - ILEB(N) + 1
          IF(IG.LT.1) THEN
           XX(I,IS) = X(I,JS)
          ELSE IF(IG.LE.NBLD(N)) THEN
           SS = SBLE(N) + STOT*SG(IG,IS)
           XX(I,IS) = SEVAL(SS,XB(1,N),XPB(1,N),SB(1,N),IIB(N))
          ELSE
           IW = I-ITEB(N)+1
           XX(I,IS) = XW(IW,N)
          ENDIF
          CP(I,IS) = (PI(I,JS) - DPINF) / QU
          CF(I,IS) = TAUW(I,IS) / QU
 4045   CONTINUE
        WRITE(LU) (XX(I,IS),CP(I,IS),THET(I,IS),DSTR(I,IS),
     &             CF(I,IS), CTAU(I,IS), I=1, II)
 404  CONTINUE
      CLOSE(LU)
C
      RETURN
      END ! PXADD

 
 
      SUBROUTINE CLRRHS(L)
      INCLUDE 'STATE.INC'
      INCLUDE 'MSES.INC'
C
      DO 1 I=1, II
        DO 10 J=1, 2*JJ-1+6*NBL
          DR(J,L,I) = 0.
   10   CONTINUE
    1 CONTINUE
C
      RETURN
      END ! CLRRHS



      SUBROUTINE SHORES(LUN)
C-----------------------------------------------
C     Dumps residuals to unit LUN in an 
C     organized fashion for debugging purposes.
C-----------------------------------------------
      INCLUDE 'STATE.INC'
      INCLUDE 'MSES.INC'
C
      N = 1
C
      ILE = ILEB(N)
      ITE = ITEB(N)
C
c      WT = 1000.0
c      DO 5 I=ITE, II-1
c        WRITE(LUN,*) ' '
c        WRITE(LUN,1000) 
c     &   I, WT*THET(I,1),WT*DSTR(I,1),WT*DISP(I,1),UEDG(I,1)
c        WRITE(LUN,1000) 
c     &   I, WT*THET(I,2),WT*DSTR(I,2),WT*DISP(I,2),UEDG(I,2)
c 1000   FORMAT(1X,I4, 5F13.8)
c    5 CONTINUE
C
      WRITE(LUN,*) ' '
      WRITE(LUN,*) ILE, ITE
      DO 10 I=1, II
CCC        IF(I.LT.ILE-1) GO TO 10
CCC        IF(I.GT.ILE+1 .AND. I.LT.ITE-3) GO TO 10
c        itr = itran(1)
c        if(.not.(i.ge.ile-3 .and. i.le.ile+3) .and.
c     &     .not.(i.ge.itr-3 .and. i.le.itr+3)) go to 10

        WRITE(LUN,*)
c        A8(JJ,I) = 1.0
        L = 5
        DO 100 J=1, JJ
          anorm = max( abs(a2(j,i))
     &               , abs(b2(j,i))
     &               , abs(c2(j,i)) )
          WRITE(LUN,1010) I,J, DR(J,L,I), DR(J+JJ,L,I)
cc     &            , q(i,j)/qstar
 1010     FORMAT(1X,2I4,2f14.7, 2E12.4, 2x, f7.3)
c          write(*,1010) i, j, a2(j,i), a8(j,i), dr(j,1,i)
c
c          WRITE(*,*) I,J, DR(J,4,I), DR(J,5,I)
c 1012     FORMAT(1x,2I4,2e13.4)
  100   CONTINUE
c        J = JS1(N)
c        WRITE(LUN,*) J+2, I, A2(J+2,I), DR(J+2,1,I), DR(J+2+JJ,1,I)
c        WRITE(LUN,*) J+1, I, A2(J+1,I), DR(J+1,1,I), DR(J+1+JJ,1,I)
c        WRITE(LUN,*) J  , I, A2(J  ,I), DR(J  ,1,I), DR(J  +JJ,1,I)
c        J = JS2(N)
c        WRITE(LUN,*) J  , I, A2(J  ,I), DR(J  ,1,I), DR(J-1+JJ,1,I)
c        WRITE(LUN,*) J-1, I, A2(J-1,I), DR(J-1,1,I), DR(J-2+JJ,1,I)
c        WRITE(LUN,*) J-2, I, A2(J-2,I), DR(J-2,1,I), DR(J-3+JJ,1,I)
        do n=1, nbl
        IS = 2*N-1
        WRITE(LUN,1110) I,DR(2*JJ  ,L,I)
        WRITE(LUN,1110) I,DR(2*JJ+1,L,I)
        WRITE(LUN,1110) I,DR(2*JJ+2,L,I)
        IS = 2*N          
        WRITE(LUN,1110) I,DR(2*JJ+3,L,I)
        WRITE(LUN,1110) I,DR(2*JJ+4,L,I)
        WRITE(LUN,1110) I,DR(2*JJ+5,L,I)
 1110   FORMAT(1X,I4,F14.3,E13.4)
        enddo
c
c        is = 0
c        DO 110 J=2*JJ, 2*JJ+6*NBL-3, 3
c          is = is+1
c          shp = 0.0
c          if(thet(i,is) .ne. 0.0) shp = dstr(i,is)/thet(i,is)
c          WRITE(LUN,1210) i, DR(J,1,I),DR(J+1,1,I),DR(J+2,1,I)
c 1210     FORMAT(1X, I4, 3f14.7, 3E12.4, 2x, f8.4)
c  110   CONTINUE
c
   10 CONTINUE
C
      RETURN
      END
